@@ -68,16 +68,43 @@ in the `auth-sources' file."
   :type 'string
   :group 'org-ai)
 
-(defcustom org-ai-chat-models '("gpt-4o-mini"
+(defcustom org-ai-chat-models '(;; "The best model for coding and agentic tasks across domains" https://platform.openai.com/docs/models/gpt-5
+                                "gpt-5.2-2025-12-11"
+                                ;; "A faster, cost-efficient version of GPT-5 for well-defined tasks" https://platform.openai.com/docs/models/gpt-5-mini
+                                "gpt-5-mini-2025-08-07"
+                                ;; "Fastest, most cost-efficient version of GPT-5" https://platform.openai.com/docs/models/gpt-5-nano
+                                "gpt-5-nano-2025-08-07"
+                                ;; Version of GPT-5 that produces smarter and more precise responses
+                                "gpt-5.2-pro-2025-12-11"
+                                ;; "Smartest non-reasoning model" https://platform.openai.com/docs/models/gpt-4.1
+                                "gpt-4.1-2025-04-14"
+                                ;; "Our most powerful deep research model" https://platform.openai.com/docs/models/o3-deep-research
+                                "o3-deep-research-2025-06-26"
+                                ;; "Faster, more affordable deep research model" https://platform.openai.com/docs/models/o4-mini-deep-research
+                                "o4-mini-deep-research-2025-06-26"
+                                ;; "GPT-5 model used in ChatGPT" https://platform.openai.com/docs/models/gpt-5-chat-latest
+                                "gpt-5-chat-latest"
+                                "gpt-4o-mini"
                                 "gpt-4"
                                 "gpt-4-32k"
                                 "gpt-4-turbo"
                                 "gpt-4o"
+                                "gpt-4o-mini"
+                                "gpt-4o-realtime-preview"
+                                "gpt-4o-search-preview"
+                                "gpt-4o-mini-search-preview"
+                                "gpt-4.1"
+                                "gpt-4.1-nano"
+                                "gpt-4.1-mini"
                                 "gpt-3.5-turbo"
                                 "o1"
+                                "o1-pro"
                                 "o1-preview"
                                 "o1-mini"
+                                "o3"
                                 "o3-mini"
+                                "o3-pro"
+                                "o4-mini"
                                 "chatgpt-4o-latest")
   "Alist of available chat models. See https://platform.openai.com/docs/models."
   :type '(alist :key-type string :value-type string)
@@ -135,18 +162,36 @@ messages."
   "Map a service name such as 'openai' to a valid `org-ai-service' symbol."
   (intern-soft name))
 
+(defun org-ai--service-of-model (model)
+  "Return the service of the model.
+`MODEL' is the model name."
+  (cond
+   ((string-prefix-p "gpt-" model) 'openai)
+   ((string-prefix-p "chatgpt-" model) 'openai)
+   ((string-prefix-p "o1" model) 'openai)
+   ((string-prefix-p "o3" model) 'openai)
+   ((string-prefix-p "o4" model) 'openai)
+   ((string-prefix-p "claude" model) 'anthropic)
+   ((string-prefix-p "gemini" model) 'google)
+   ((string-prefix-p "deepseek" model) 'deepseek)
+   (t nil)))
+
 (defcustom org-ai-service 'openai
-  "Service to use. Either openai or azure-openai."
+  "Service to use."
   :type '(choice (const :tag "OpenAI" openai)
                  (const :tag "Azure-OpenAI" azure-openai)
                  (const :tag "perplexity.ai" perplexity.ai)
-                 (const :tag "anthropic" anthropic))
+                 (const :tag "anthropic" anthropic)
+                 (const :tag "DeepSeek" deepseek)
+                 (const :tag "google" google))
   :group 'org-ai)
 
 (defvar org-ai-openai-chat-endpoint "https://api.openai.com/v1/chat/completions")
 
 (defvar org-ai-openai-completion-endpoint "https://api.openai.com/v1/completions")
 
+(defvar org-ai-google-chat-endpoint "https://generativelanguage.googleapis.com/v1beta/openai/chat/completions")
+
 ;; Azure-Openai specific variables
 
 (defcustom org-ai-azure-openai-api-base "https://your-instance.openai.azure.com"
@@ -188,8 +233,10 @@ Either from `org-ai-openai-api-token' or from auth-source."
   (let* ((service (or service org-ai-service))
          (endpoint (pcase service
                      ('openai "api.openai.com")
+                     ('deepseek "api.deepseek.com")
                      ('perplexity.ai "api.perplexity.ai")
                      ('anthropic "api.anthropic.com")
+                     ('google "generativelanguage.googleapis.com")
                      ('azure-openai (strip-api-url org-ai-azure-openai-api-base)))))
     (or (auth-source-pick-first-password :host endpoint :user "org-ai")
         (auth-source-pick-first-password :host endpoint :login "org-ai"))))
@@ -214,8 +261,12 @@ whether messages are provided."
 	      (if messages "/chat" "") org-ai-azure-openai-api-version))
      ((eq service 'perplexity.ai)
       "https://api.perplexity.ai/chat/completions")
+     ((eq service 'deepseek)
+      "https://api.deepseek.com/v1/chat/completions")
      ((eq service 'anthropic)
       "https://api.anthropic.com/v1/messages")
+     ((eq service 'google)
+      org-ai-google-chat-endpoint)
      (t
       (if messages org-ai-openai-chat-endpoint org-ai-openai-completion-endpoint)))))
 
@@ -229,6 +280,9 @@ whether messages are provided."
         ((eq service 'anthropic)
          `(("x-api-key" . ,(org-ai--openai-get-token service))
            ("anthropic-version" . ,org-ai-anthropic-api-version)))
+        ((eq service 'google)
+         `(("Accept-Encoding" . "identity")
+           ("Authorization" . ,(encode-coding-string (string-join `("Bearer" ,(org-ai--openai-get-token service)) " ") 'utf-8))))
         (t
          `(("Authorization" . ,(encode-coding-string (string-join `("Bearer" ,(org-ai--openai-get-token service)) " ") 'utf-8))))))))
 
@@ -270,6 +324,9 @@ only contain fragments.")
 (defvar org-ai--currently-inside-code-markers nil)
 (make-variable-buffer-local 'org-ai--currently-inside-code-markers)
 
+(defvar org-ai--currently-reasoning nil)
+(make-variable-buffer-local 'org-ai--currently-reasoning)
+
 (defvar org-ai--url-buffer-last-position-marker nil
   "Local buffer var to store last read position.")
 ;; (make-variable-buffer-local 'org-ai--url-buffer-last-position-marker)
@@ -382,6 +439,15 @@ from the OpenAI API."
 ;;   (type "content_block_stop" index 0)
 ;;   (type "message_delta" delta (stop_reason "end_turn" stop_sequence nil) usage (output_tokens 22))
 ;;   (type "message_stop"))
+;;
+;; and Google
+;; '((created 1745008491
+;;    model "gemini-2.5-pro-preview-03-25"
+;;    object "chat.completion.chunk"
+;;    choices [(delta (content "Hello Robert! How can I help you today?"
+;;                     role "assistant")
+;;              finish_reason "stop"
+;;              index 0)]))
 (defun org-ai--normalize-response (response)
   "This function normalizes JSON data received from OpenAI-style, Anthropic, and Perplexity endpoints.
 `RESPONSE' is one JSON message of the stream response."
@@ -441,17 +507,32 @@ from the OpenAI API."
                 (make-org-ai--response :type 'text :payload text)
                 (make-org-ai--response :type 'stop :payload finish-reason))))
 
-       ;; try openai streamed
+       ;; try openai, deepseek, gemini streamed
        (t (let ((choices (plist-get response 'choices)))
             (cl-loop for choice across choices
-                     append (or (when-let ((role (plist-get (plist-get choice 'delta) 'role)))
-                                  (list (make-org-ai--response :type 'role :payload role)))
-				(when-let ((role (plist-get (plist-get choice 'delta) 'content)))
-                                  (list (make-org-ai--response :type 'text :payload role)))
-                                (when-let ((content (plist-get (plist-get choice 'delta) 'content)))
-                                  (list (make-org-ai--response :type 'text :payload content)))
-                                (when-let ((finish-reason (plist-get choice 'finish_reason)))
-                                  (list (make-org-ai--response :type 'stop :payload finish-reason)))))))))))
+                     append (let* ((delta (plist-get choice 'delta))
+                                   (role (when-let ((role (plist-get delta 'role)))
+                                           (if (and (string= "assistant" role)
+                                                    (plist-get delta 'reasoning_content))
+                                               "assistant_reason"
+                                             role)))
+                                   (content (plist-get (plist-get choice 'delta) 'content))
+                                   (reasoning-content (plist-get delta 'reasoning_content))
+                                   (finish-reason (plist-get choice 'finish_reason))
+                                   (result nil))
+                              (when finish-reason
+                                (push (make-org-ai--response :type 'stop :payload finish-reason) result))
+                              (when reasoning-content
+                                (setq org-ai--currently-reasoning t)
+                                (push (make-org-ai--response :type 'text :payload reasoning-content) result))
+                              (when (and content (> (length content) 0))
+                                (push (make-org-ai--response :type 'text :payload content) result)
+                                (when org-ai--currently-reasoning
+                                  (setq org-ai--currently-reasoning nil)
+                                  (push (make-org-ai--response :type 'role :payload "assistant") result)))
+                              (when role
+                                (push (make-org-ai--response :type 'role :payload role) result))
+                              result))))))))
 
 (defun org-ai--insert-stream-response (context buffer &optional response insert-role)
   "`RESPONSE' is one JSON message of the stream response.
@@ -487,6 +568,8 @@ the response into."
                                    (setq org-ai--current-chat-role role)
                                    (let ((role (and insert-role (org-ai--response-payload response))))
                                      (cond
+                                      ((string= role "assistant_reason")
+                                       (insert "\n[AI_REASON]: "))
                                       ((string= role "assistant")
                                        (insert "\n[AI]: "))
                                       ((string= role "user")
@@ -547,6 +630,7 @@ penalty. `PRESENCE-PENALTY' is the presence penalty."
   (setq org-ai--debug-data-raw nil)
   (setq org-ai--currently-inside-code-markers nil)
   (setq service (or (if (stringp service) (org-ai--read-service-name service) service)
+                    (org-ai--service-of-model model)
                     org-ai-service))
   (setq stream (org-ai--stream-supported service model))
 
@@ -751,7 +835,8 @@ and the length in chars of the pre-change text replaced by that range."
 `SERVICE' is the service to use. `MODEL' is the model to use."
   ;; stream not supported by openai o1 models
   (not (and (or (eq service 'openai) (eq service 'azure-openai))
-            (or (string-prefix-p "o1" model) (string-prefix-p "o3" model)))))
+            (or
+             (string-prefix-p "o1-pro" model)))))
 
 (defun org-ai-interrupt-current-request ()
   "Interrupt the current request."
@@ -821,7 +906,12 @@ and the length in chars of the pre-change text replaced by that range."
 (defun org-ai-switch-chat-model ()
   "Change `org-ai-default-chat-model'."
   (interactive)
-  (let ((model (completing-read "Model: " org-ai-chat-models nil t)))
+  (let ((model (completing-read "Model: "
+                                (append org-ai-chat-models
+                                        '("claude-3-opus-latest" "claude-3-5-sonnet-latest" "claude-3-7-sonnet-latest"
+                                          "gemini-2.5-pro-preview-03-25" "gemini-2.5-flash-preview-04-17" "gemini-2.0-flash" "gemini-2.0-pro-exp"
+                                          "deepseek-chat" "deepseek-reasoner"))
+                                nil t)))
     (setq org-ai-default-chat-model model)))
 
 ;; -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
